searchData={"items":[{"type":"module","title":"Caravan","doc":"Tools for running Distributed Elixir/Erlang with Nomad and Consul\n\n# Caravan\n\nThe built-in Erlang distribution mechanisms are prefaced on using the Erlang\nPort Mapper Daemon(EPMD), a process that is started with the VM and communicates with\nremote EPMD instances to determine what ports to send data on.\nWhile this method can work in some cloud environments, container scheduling technologies make it\ndifficult to pick a single port to use globally or to run multiple processes in a continer.\nAlso, the built-in method for forming a cluster is to use a plaintext .hosts file with\nresolvable node names, which is very difficult to make work in a dynamic environment where\nnodes can leave a cluster frequently.\n\nThere are several libraries and strategies for using the Kubernetes API to\nbuild a distributed cluster, but Consul provides us with a clean DNS api to\nretrieve information, while Nomad handles monitoring and scheduling services.\n\nCaravan is split into two parts: The first is a set of modules that remove the\nneed for `epmd` by determing node ports by the node name. The idea and much of\nthe code is from the excellent article [Erlang (and Elixir) distribution\nwithout\nepmd](https://www.erlang-solutions.com/blog/erlang-and-elixir-distribution-without-epmd.html).\nIt's worth the read, and should explain what we're trying to accomplish with\nthe `Caravan.Epmd` module.\n\nThe second part utilizes [libcluster](https://github.com/bitwalker/libcluster)\nto help with forming clusters automatically based on DNS SRV queries to\nConsul.","ref":"Caravan.html"},{"type":"module","title":"Getting started with custom Erlang distribution - Caravan","doc":"Erlang has some command line options to overwrite the default distribution\nmechanism. To use Caravan's implementations, you would do something similar to\nthis\n```\niex --erl \"-proto_dist Elixir.Caravan.Epmd.Dist -start_epmd false -epmd_module Elixir.Caravan.Epmd.Client\" --sname \"node3434\" -S mix\n```\nFor testing locally, you'll either have to run `elixirc` on the above\nmodules to create the required `.beam` files, or you can pass an additional\nflag to `--erl`:\n```\n-pa _build/dev/lib/caravan/ebin\n```\nNote: building a release with Distillery will not require the `-pa` flag.","ref":"Caravan.html#module-getting-started-with-custom-erlang-distribution"},{"type":"module","title":"Caravan.Cluster.Config","doc":"Config for `Caravan.Cluster.DnsStrategy.`\n\n  - topology: topology name passed to `Cluster.Strategy.connect_nodes/4`\n  - query: The name to query for SRV records. Something like:  `prod-likes-service-dist-consul`\n  - dns_client: module implementing `Caravan.DnsClient`. Defaults to `Caravan.DnsClient.InetRes`.\n  - node_sname: the base of a node name. App name is a good candidate.\n  - connect: Override to use a different transport mechanism. Uses `:libcluster` defaults otherwise.\n  - disconnect: Override to use a different transport mechanism. Uses `:libcluster` defaults otherwise.\n  - list_nodes: Override to use a different transport mechanism. Uses `:libcluster` defaults otherwise.\n  - poll_interval: poll the dns server on this interval. Defaults to `5_000`","ref":"Caravan.Cluster.Config.html"},{"type":"function","title":"Caravan.Cluster.Config.new/1","doc":"Takes a `Cluster.Strategy.State` and returns a Config struct","ref":"Caravan.Cluster.Config.html#new/1"},{"type":"type","title":"Caravan.Cluster.Config.t/0","doc":"","ref":"Caravan.Cluster.Config.html#t:t/0"},{"type":"module","title":"Caravan.Cluster.DnsStrategy","doc":"Implements a libcluster strategy for node distribution based on Consul DNS.  By\ndefault it uses `:inet_res` to query the nameservers, though it can be configured to use any\nmodule conforming to the `Caravan.DnsClient` behaviour.","ref":"Caravan.Cluster.DnsStrategy.html"},{"type":"module","title":"Prerequisites - Caravan.Cluster.DnsStrategy","doc":"First things first, is that you'll need to have Consul setup and configured as a nameserver either\nin your hosts file or via an [erl_inetrc file](http://erlang.org/doc/apps/erts/inet_cfg.html) You'll\nneed to create a service that will return SRV records with the hostname and\ndistribution port. The [Consul documentation](https://www.consul.io/docs/agent/dns.html#standard-lookup)\nhas directions on what needs to be setup and how to test with `dig`.\n\nLet's look at an example:\n```\n'likes-service-3434@prod.socialmedia.consul`\n```\nAbove, `likes-service` is an app name. It will correspond with the :node_sname\nconfig option. The port is the last integer to the left of the '@'. You'll\nneed this because our nodes will be using `Caravan.Epmd.Client` and\n`Caravan.Epmd.Dist_dist` to use the port number of the node name instead of\nbeing assigned a port randomly by `epmd`.\n\nAlso note that the hostname of cluster nodes returned by Consul must be the\nsame as that in the nodes `-name` parameter","ref":"Caravan.Cluster.DnsStrategy.html#module-prerequisites"},{"type":"module","title":"Configuration - Caravan.Cluster.DnsStrategy","doc":"Here's a sample configuration\n```\nconfig :libcluster,\n  topologies: [\n    caravan: [\n      # The selected clustering strategy. Required.\n      strategy: Caravan.Cluster.DnsStrategy,\n      config: [\n        #service name that returns the distribution port in a SRV record\n        query: \"likes-service-dist.service.consul\",\n        #forms the base of the node name. App name is a good one.\n        node_sname: \"profile-service\",\n        #The poll interval for the Consul service in milliseconds. Defaults to 5s\n        poll_interval: 5_000\n        #The module of the DNS client to use.\n        dns_client: Caravan.DnsClient\n      ],\n    ]\n  ]\n```","ref":"Caravan.Cluster.DnsStrategy.html#module-configuration"},{"type":"function","title":"Caravan.Cluster.DnsStrategy.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Caravan.Cluster.DnsStrategy.html#child_spec/1"},{"type":"function","title":"Caravan.Cluster.DnsStrategy.find_nodes/1","doc":"","ref":"Caravan.Cluster.DnsStrategy.html#find_nodes/1"},{"type":"behaviour","title":"Caravan.DnsClient","doc":"A client for fetching SRV records from DNS and returning.","ref":"Caravan.DnsClient.html"},{"type":"callback","title":"Caravan.DnsClient.get_nodes/1","doc":"Accepts a valid dns record name. Returns a list of {port, host} which can then be used to\nconstruct a node name.","ref":"Caravan.DnsClient.html#c:get_nodes/1"},{"type":"module","title":"Caravan.Epmd","doc":"Shared logic for determining the Erlang distribution port from the node name.","ref":"Caravan.Epmd.html"},{"type":"function","title":"Caravan.Epmd.dist_port/1","doc":"Returns the Erlang Distribution port based on a node name.","ref":"Caravan.Epmd.html#dist_port/1"},{"type":"function","title":"Examples - Caravan.Epmd.dist_port/1","doc":"iex> Caravan.Epmd.dist_port(:\"app-name3434@example.net\")\n    3434\n\n    iex> Caravan.Epmd.dist_port(:\"app-name-4834@example.net\")\n    4834","ref":"Caravan.Epmd.html#dist_port/1-examples"},{"type":"module","title":"Caravan.Epmd.Client","doc":"Implementation of the `epmd` client logic. Meant for use with the `-epmd_module`\nflag\n\nIt will return a port from `Caravan.Epmd.dist_port/1` as opposed to calling\nout to the `epmd` daemon and having it assign us one.","ref":"Caravan.Epmd.Client.html"},{"type":"function","title":"Caravan.Epmd.Client.address_please/3","doc":"","ref":"Caravan.Epmd.Client.html#address_please/3"},{"type":"function","title":"Caravan.Epmd.Client.names/1","doc":"","ref":"Caravan.Epmd.Client.html#names/1"},{"type":"function","title":"Caravan.Epmd.Client.port_please/2","doc":"","ref":"Caravan.Epmd.Client.html#port_please/2"},{"type":"function","title":"Caravan.Epmd.Client.register_node/2","doc":"","ref":"Caravan.Epmd.Client.html#register_node/2"},{"type":"function","title":"Caravan.Epmd.Client.register_node/3","doc":"","ref":"Caravan.Epmd.Client.html#register_node/3"},{"type":"function","title":"Caravan.Epmd.Client.start_link/0","doc":"","ref":"Caravan.Epmd.Client.html#start_link/0"},{"type":"module","title":"Caravan.Epmd.Dist_dist","doc":"Implements the Erlang distribution protocol. Forwards most calls to\n`:inet_tcp_dist` with the exception of `listen/1` which uses `Caravan.Epmd`\nto get the distribution port instead of calling out to `epmd`. The `_dist`\npart of the module name is required for it to work, but will be used like this\n`-proto_dist Caravan.Epmd.Dist`","ref":"Caravan.Epmd.Dist_dist.html"},{"type":"function","title":"Caravan.Epmd.Dist_dist.accept/1","doc":"","ref":"Caravan.Epmd.Dist_dist.html#accept/1"},{"type":"function","title":"Caravan.Epmd.Dist_dist.accept_connection/5","doc":"","ref":"Caravan.Epmd.Dist_dist.html#accept_connection/5"},{"type":"function","title":"Caravan.Epmd.Dist_dist.close/1","doc":"","ref":"Caravan.Epmd.Dist_dist.html#close/1"},{"type":"function","title":"Caravan.Epmd.Dist_dist.is_node_name/1","doc":"","ref":"Caravan.Epmd.Dist_dist.html#is_node_name/1"},{"type":"function","title":"Caravan.Epmd.Dist_dist.listen/1","doc":"","ref":"Caravan.Epmd.Dist_dist.html#listen/1"},{"type":"function","title":"Caravan.Epmd.Dist_dist.select/1","doc":"","ref":"Caravan.Epmd.Dist_dist.html#select/1"},{"type":"function","title":"Caravan.Epmd.Dist_dist.setup/5","doc":"","ref":"Caravan.Epmd.Dist_dist.html#setup/5"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}